
package org.atlas.desk.gui;

import org.atlas.desk.events.DisplayEvent;
import org.atlas.desk.events.DisplayListener;
import org.atlas.desk.events.FeatureInfoEvent;
import org.atlas.desk.events.FeatureInfoListener;
import org.atlas.desk.events.GeocodingEvent;
import org.atlas.desk.events.GeocodingListener;
import org.atlas.desk.events.HttpRequestEvent;
import org.atlas.desk.events.HttpRequestListener;
import org.atlas.desk.events.MarkerClickedEvent;
import org.atlas.desk.events.MarkerClickedListener;
import org.atlas.desk.events.MouseClickEvent;
import org.atlas.desk.events.MouseClickListener;
import org.atlas.desk.events.MouseMoveEvent;
import org.atlas.desk.events.MouseMoveListener;
import org.atlas.geocoding.Result;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import org.atlas.desk.mashup.Marker;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.atlas.util.CoordinateUtils;

/**
 * 
 * @author Mario
 */
public class EventManager {

    private List<MarkerClickedListener> markerClickedListeners;
    private List<MouseClickListener>    mouseClickListeners;
    private List<MouseMoveListener>     mouseMoveListeners;
    private List<DisplayListener>       displayListeners;
    private List<FeatureInfoListener>   featureInfoListeners;
    private List<GeocodingListener>     geocodingListeners;
    private List<HttpRequestListener>   httpRequestListeners;
        
    private MapPanel map;

    /**
     * Clase que funciona en conjunto con MapPanel y le permite administrar los escuchadores
     * a diferentes eventos así como dispararlos.
     * @param map
     */
    public EventManager(MapPanel map) {
        markerClickedListeners = new ArrayList<MarkerClickedListener>();
        mouseClickListeners    = new ArrayList<MouseClickListener>();
        mouseMoveListeners     = new ArrayList<MouseMoveListener>();
        displayListeners       = new ArrayList<DisplayListener>();
        featureInfoListeners   = new ArrayList<FeatureInfoListener>();
        httpRequestListeners   = new ArrayList<HttpRequestListener>();
        geocodingListeners     = new ArrayList<GeocodingListener>();
        this.map = map;
    }

    //FEATUREINFO LISTENERS
    /**
     * Agrega un escuchador de peticiones FeatureInfo.
     * @param fl
     */
    public void addFeatureInfoListener(FeatureInfoListener fl) {
        featureInfoListeners.add(fl);
    }

    /**
     * Remueve un escuchador de peticiones FeatureInfo. 
     * @param fl
     */
    public void removeFeatureInfoListener(FeatureInfoListener fl) {
        featureInfoListeners.remove(fl);
    }
    
    protected void fireFeatureInfoEvent(String info, String url) {
        FeatureInfoEvent f = new FeatureInfoEvent(map, info, url, 0);
        for (int i = 0; i < featureInfoListeners.size(); i++) {
            ((FeatureInfoListener) featureInfoListeners.get(i)).actionPerformed(f);
        }
    }

    //MOUSE CLICK LISTENERS
    /**
     * Agrega un escuchador de clic de mouse en el mapa.
     * @param l
     */
    public void addMouseClickListener(MouseClickListener l) {
        mouseClickListeners.add(l);
    }

    /**
     * Remueve un escuchador de clic de mouse en el mapa. 
     * @param l
     */
    public void removeMouseClickListener(MouseClickListener l) {
        mouseClickListeners.remove(l);
    }
    
    protected void fireMouseClickEvent(MouseEvent me) {
        if (map.getVisibleBbox() != null) {
            Point2D xy = CoordinateUtils.translateIJtoPoint(map.getVisibleBbox(), map.getWidth(), map.getHeight(), me.getX(), me.getY(), map.getCurrentProject().getCrs());
            MouseClickEvent mme = new MouseClickEvent(map, -1, new Point2D.Double(me.getX(), me.getY()), xy);
            for (int i = 0; i < mouseClickListeners.size(); i++) {
                ((MouseClickListener) mouseClickListeners.get(i)).actionPerformed(mme);
            }
        }
    }

    //MARKER CLICK LISTENERS
    /**
     * Agrega un escuchador de clic de mouse en un marcador. 
     * @param mcl
     */
    public void addMarkerClickedListener(MarkerClickedListener mcl) {
        markerClickedListeners.add(mcl);
    }

    /**
     * Remueve un escuchador de clic de mouse en un marcador.  
     * @param mcl
     */
    public void removeMarkerClickedListener(MarkerClickedListener mcl) {
        markerClickedListeners.remove(mcl);
    }

    
    protected void fireMarkerClickedEvent(MouseEvent me, Marker marker) {
        if (map.getVisibleBbox() != null) {
            Point2D xy = CoordinateUtils.translateIJtoPoint(map.getVisibleBbox(), map.getWidth(), map.getHeight(), me.getX(), me.getY(), map.getCurrentProject().getCrs());
            MarkerClickedEvent mce = new MarkerClickedEvent(map, -1, new Point2D.Double(me.getX(), me.getY()), xy, marker);
            for (int i = 0; i < markerClickedListeners.size(); i++) {
                markerClickedListeners.get(i).actionPerformed(mce);
            }
        }
    }
        
    protected void fireMarkerClickedEvent(Marker marker) {
        if (map.getVisibleBbox() != null) {
            MarkerClickedEvent mce = new MarkerClickedEvent(map, -1, null, null, marker);
            for (int i = 0; i < markerClickedListeners.size(); i++) {
                markerClickedListeners.get(i).actionPerformed(mce);
            }
        }
    }

    
    //MOUSE MOVE LISTENERS
    /**
     * Agrega un escuchador de movimiento de mouse en el mapa. 
     * @param ml
     */
    public void addMouseMoveListener(MouseMoveListener ml) {
        mouseMoveListeners.add(ml);
    }

    /**
     * Remueve un escuchador de movimiento de mouse en el mapa.  
     * @param al
     */
    public void removeMouseMoveListener(MouseMoveListener al) {
        mouseMoveListeners.remove(al);
    }

    
    protected void fireMouseMoveEvent(MouseEvent me) {
        if (map.getVisibleBbox() != null) {
            Point2D xy = CoordinateUtils.translateIJtoPoint(map.getVisibleBbox(), map.getWidth(), map.getHeight(), me.getX(), me.getY(), map.getCurrentProject().getCrs());
            MouseMoveEvent mme = new MouseMoveEvent(map, xy, new Point2D.Double(me.getX(), me.getY()), 0);
            for (int i = 0; i < mouseMoveListeners.size(); i++) {
                ((MouseMoveListener) mouseMoveListeners.get(i)).actionPerformed(mme);
            }
        }
    }

    //DISPLAY LISTENERS
    /**
     * Agrega un escuchador de movimiento de mouse en el mapa. 
     * @param l
     */
    public void addDisplayListener(DisplayListener l) {
        displayListeners.add(l);
    }

    /**
     * Remueve un escuchador de movimiento de mouse en el mapa. 
     * @param l
     */
    public void removeDisplayMoveListener(DisplayListener l) {
        displayListeners.remove(l);
    }

    
    protected void fireDisplayEvent() {
        if (map.getVisibleBbox() != null) {
            DisplayEvent ev = new DisplayEvent(map, map.getVisibleBbox(), map.getCurrentProject().getCrs(), 0);
            for (int i = 0; i < displayListeners.size(); i++) {
                ((DisplayListener) displayListeners.get(i)).actionPerformed(ev);
            }
        }
    }

    //HTTPREQUEST LISTENERS
    /**
     * 
     * @param l
     */
    public void addHttpRequestListener(HttpRequestListener l) {
        httpRequestListeners.add(l);
    }

    /**
     * 
     * @param l
     */
    public void removeHttpRequestListener(HttpRequestListener l) {
        httpRequestListeners.remove(l);
    }

    /**
     * 
     * @param url
     * @param status
     */
    public void fireHttpRequestEvent(URL url, int status) {
        HttpRequestEvent ev = new HttpRequestEvent(map, url.toString(), 0);
        for (int i = 0; i < httpRequestListeners.size(); i++) {
            ((HttpRequestListener) httpRequestListeners.get(i)).actionPerformed(ev);
        }
    }

    
    protected void fireHttpRequestEvent(HttpRequestEvent ev) {
        for (int i = 0; i < httpRequestListeners.size(); i++) {
            ((HttpRequestListener) httpRequestListeners.get(i)).actionPerformed(ev);
        }
    }

    //GEOCODING LISTENERS
    /**
     * Agrega un escuchador de respuestas del geocoder.
     * @param gl
     */
    public void addGeocodingListener(GeocodingListener gl) {
        geocodingListeners.add(gl);
    }

    /**
     * Remueve un escuchador de respuestas del geocoder. 
     * @param gl
     */
    public void removeGeocodingListener(GeocodingListener gl) {
        geocodingListeners.remove(gl);
    }

    
    protected void fireGeocodingEvent(byte[] buf) throws JDOMException, IOException {

        SAXBuilder builder = new SAXBuilder();
        Document doc = builder.build(new ByteArrayInputStream(buf));
        Element response = (Element) doc.getRootElement();
        String request = response.getAttribute("request").getValue();
        String prjName = response.getAttribute("project").getValue();
        Element coordinates = (Element) response.getChildren().get(0);
        Result[] rs = new Result[coordinates.getChildren().size()];
        for (int i = 0; i < coordinates.getChildren().size(); i++) {
            Element coordinate = (Element) coordinates.getChildren().get(i);
            Point2D p = new Point2D.Double(Double.parseDouble(coordinate.getAttributeValue("x")), Double.parseDouble(coordinate.getAttributeValue("y")));
            rs[i] = new Result(coordinate.getAttributeValue("address"), p);
        }
        
        GeocodingEvent ev = new GeocodingEvent(map, prjName, request, rs);
        for (int i = 0; i < geocodingListeners.size(); i++) {
            ((GeocodingListener) geocodingListeners.get(i)).actionPerformed(ev);
        }
    }
}
