
package org.atlas.geocoding;

import org.atlas.hibernatedata.GeocoderConfig;
import org.atlas.hibernatedata.Project;
import java.util.regex.Pattern;
import javax.swing.JDialog;
import org.atlas.util.ConnectionProperties;
import org.atlas.util.HibernateUtil;

/**
 * Esta es la clase que deben extender todos los geocoders
 * @author Mario
 */
public abstract class Geocoder {
    
    /**
     * Indica si el geocoder usa o no la fuente con identificacion idsour.
     * @return
     */
    public abstract boolean isUsingSource(int idsour);
    
    /**
     * Retorna el proyecto para el que funcionará esta instancia del geocoder
     * @return
     */
    public abstract Project getProject();
    
    /**
     * Establece el proyecto para el que funcionará esta instancia del geocoder. Normalmente solo es llamado por el gestór
     * @param project
     */
    public abstract void setProject(Project project);
            
    
    /**
     * Establece el objeto de configuración del geocoder para el proyecto. Normalmente solo es llamado por el gestór
     * @param gc
     */
    public abstract void setGeocoderConfig(GeocoderConfig gc);
    
    
    /**
     * Es llamado por el gestór en servidor antes de empezar a hacer peticiones, aquí el geocoder debe realizar todas 
     * las acciones que sean necesarias antes de empezar a operar, como por ejemplo, formar caches y verificar el estado de las tablas requeridas
     * @param c
     * @throws java.lang.Exception
     */
    public abstract void init(ConnectionProperties c) throws Exception;
    
    
    /**
     * El nombre del geocoder para ser mostrado en pantalla
     * @return
     */
    public abstract String getName();
    
    /**
     * Una descripción del geocoder para ser mostrada en pantalla
     * @return
     */
    public abstract String getDescription();
    
    
    /**
     * Debe retornar un dialog de configuración listo para funcionar con los objetos que se entregan como parámetros
     * @param hibernate
     * @param project
     * @param config
     * @return
     */
    public abstract ConfigurationDialog getConfigurationDialog(HibernateUtil hibernate, Project project, GeocoderConfig config);
    
    /**
     * Debe retornar un cuadro de dialogo con información sobre el geocoder
     * @return
     */
    public abstract JDialog getAboutDialog();
        
    /**
     * Debe retornar falso o verdadero en función de si el geocoder puede o no procesar 
     * la dirección indicada, dicha dirección se entrega exactamente como llega en la petición.
     * @param address
     * @return
     * @throws java.lang.Exception
     */
    public abstract boolean isCapableFor(String address) throws Exception;

    
    /**
     * Retorna la dirección indicada después de llevar a una forma en que contiene
     * elementos lexicos estandarizados para el proceso, por ejemplo, el reemplazo de 
     * "calle", "cll", "clle", "cl" por "CL".
     * @param address
     * @return
     */
    public abstract String getStandardizedAddress(String address);
    
    
    /**
     * Retorna los componentes léxicos de la dirección estandarizada que se entrega como parámetro.
     * @param adress
     * @return
     * @throws java.lang.Exception 
     */
    public abstract String[] parseAdress(String adress) throws Exception;
    
    
    
    /**
     * Determina las posibles ubicaciones para una dirección dados sus componentes léxicos
     * @param parts
     * @return
     * @throws java.lang.Exception 
     */
    public abstract Result[] locateAddress(String[] parts) throws Exception;
     
    
    /**
     * Retorna una cadena en la que se han suprimido todos los signos de puntación y se ha removido 
     * el acento de todas la vocales, tambien se han eliminado palabras como 
     * CON, DE, EL, EN, LA, LAS, LO, LOS, MAS, MI, MIS, PARA, POR, QUE, SIN, SU, SUS, TU, TUS, YA, y se
     * ha convertido a mayusculas
     * @param address
     * @return
     */
    public static String getCleanAddress(String address) {
        String res = new String(address);
        res = res.toUpperCase();
        res = res.replaceAll("[Á]", "A");
        res = res.replaceAll("[É]", "E");
        res = res.replaceAll("[Í]", "I");
        res = res.replaceAll("[Ó]", "O");
        res = res.replaceAll("[Ú]", "U");
        res = res.replaceAll("\\b(CON|DE|EL|EN|LA|LAS|LO|LOS|MAS|MI|MIS|PARA|POR|QUE|SIN|SU|SUS|TU|TUS|YA)\\b", " ");
        res = res.replaceAll("[\\.,;:\\-_]", " ");
        return res;
    }

    
    /**
     * Retorna una cadena en la que todas las ocurrencias de varios espacios consecutivos
     * se han reemplazado por un solo espacio
     * @param address dirección que se desea procesar
     * @return dirección procesada
     */
    public static String getSingleSpaced(String address) {
        String res = new String(address);
        res = res.toUpperCase();
        res = res.replaceAll("[\\s]{2,}", " ");
        return res.trim();
    }

    
    /**
     * Busca una cadena en otra sin tener en cuenta mayusculas o minusculas
     * @param pattern lo que se desea buscar
     * @param target la cadena en que se busca
     * @return
     */
    protected static boolean iFind(String pattern, String target){
        return Pattern.compile(pattern, Pattern.CASE_INSENSITIVE).matcher(target).find();
    }
    
    @Override
    public String toString(){
        return getName();
    }
    
}
