package org.atlas.hibernatedata;

import org.atlas.geocoding.Geocoder;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import org.atlas.util.BoundingBox;
import org.jdom.Element;
import org.jdom.Namespace;


/**
 * Representa un proyecto para su uso en el servidor y en el panel de control. Soporta persistencia
 * @author Mario
 */

public class Project implements Comparable {

    private int codeproj;
    private String descproj;
    private Spatial_ref_sys spatial_ref_sys;
    private Service service;
    private Set layers = new HashSet();
    private Set sources = new HashSet();
    private Set geocoderConfigs = new HashSet();
    private Geocoder[] geocoders = new Geocoder[0];

    /**
     * Construye un objeto en blanco
     */
    public Project() {
    }

    /**
     * Carga los geocoders asignados al proyecto, solicita las operaciones requeridas al 
     * cargador de clases del sistema, requiere que los plugins de los geocoders 
     * esten presentes.
     * 
     * @throws java.lang.ClassNotFoundException
     * @throws java.lang.InstantiationException
     * @throws java.lang.IllegalAccessException
     */
    public void loadGeocoders() throws ClassNotFoundException, InstantiationException, IllegalAccessException{
        geocoders = new Geocoder[geocoderConfigs.size()];
        Object[] obj = geocoderConfigs.toArray();
        for (int i = 0; i < obj.length; i++) {
            GeocoderConfig conf = (GeocoderConfig) obj[i];
            Class geoclass = Class.forName(conf.getClassname());
            geocoders[i] = (Geocoder) geoclass.newInstance();
            geocoders[i].setGeocoderConfig(conf);
            geocoders[i].setProject(this);
        }
    }

    /**
     * Código del objeto en el sistema de persistencia
     * @return
     */
    public int getCodeproj() {
        return codeproj;
    }

    /**
     * Código del objeto en el sistema de persistencia 
     * @param codeproj
     */
    public void setCodeproj(int codeproj) {
        this.codeproj = codeproj;
    }

    /**
     * Nombre que mostrar el proyecto.
     * @return
     */
    public String getDescproj() {
        return descproj;
    }

    /**
     * Nombre que mostrar el proyecto. 
     * @param descproj
     */
    public void setDescproj(String descproj) {
        this.descproj = descproj;
    }

    /**
     * Sistema de referencia espacial del proyecto
     * @return
     */
    public Spatial_ref_sys getSpatial_ref_sys() {
        return spatial_ref_sys;
    }

    /**
     * Sistema de referencia espacial del proyecto 
     * @param spatial_ref_sys
     */
    public void setSpatial_ref_sys(Spatial_ref_sys spatial_ref_sys) {
        this.spatial_ref_sys = spatial_ref_sys;
    }

    /**
     * Conjuto de capas asociadas al sistema
     * @return
     */
    public Set getLayers() {
        return layers;
    }

    /**
     * Conjuto de capas asociadas al sistema 
     * @param layers
     */
    public void setLayers(Set layers) {
        this.layers = layers;
    }

    /**
     * Identificador del proyecto según el estándar WMS
     * @return
     */
    public String getNameProj() {
        return "p" + getCodeproj();
    }

    @Override
    public String toString() {
        return this.getDescproj();
    }

    public int compareTo(Object o) {
        Project p = (Project) o;
        String a = p.toString();
        String b = this.toString();
        return b.compareTo(a);
    }

    /**
     * Servicio al que pertenece el proyecto
     * @return
     */
    public Service getService() {
        return service;
    }

    /**
     * Servicio al que pertenece el proyecto 
     * @param service
     */
    public void setService(Service service) {
        this.service = service;
    }

    /**
     * Listado de las capas del proyecto ordenadas según su Z-order
     * @return
     */
    public ArrayList<Layer> getSortedLayers() {
        ArrayList<Layer> zlayers = new ArrayList(this.getLayers());
        Collections.sort(zlayers, new Comparator() {

            public int compare(Object o1, Object o2) {
                int v1, v2;
                v1 = ((Layer) o1).getZorder();
                v2 = ((Layer) o2).getZorder();
                if (v1 < v2) {
                    return -1;
                }
                if (v1 == v2) {
                    return 0;
                }
                return 1;
            }
        });
        return zlayers;
    }

    /**
     * Obtiene un elemento XML que describe el proyecto y sus capas
     * @param root
     * @param version
     * @return
     */
    public Element getXMLCapability(Element root, int version) {
        Namespace ns = root.getNamespace();
        try {
            String CRS = "EPSG:" + this.getSpatial_ref_sys().getSrid();
            Iterator<Layer> ite = this.getSortedLayers().iterator();
            Element project = new Element("Layer", ns);
            project.addContent(new Element("Name", ns).setText(this.getNameProj()));
            project.addContent(new Element("Title", ns).setText(this.getDescproj()));
            String path = "";
            switch (version) {
                case Service.VERSION_1_1_0:
                    project.addContent(new Element("SRS", ns).setText(CRS));
                    path = "/xml/GetLayer_1.1.0.xml";
                    break;
                case Service.VERSION_1_1_1:
                    project.addContent(new Element("SRS", ns).setText(CRS));
                    path = "/xml/GetLayer_1.1.1.xml";
                    break;
                case Service.VERSION_1_3_0:
                    project.addContent(new Element("CRS", ns).setText(CRS));
                    path = "/xml/GetLayer_1.3.0.xml";
                    break;
            }
            project.addContent(this.getBoundingBox().getXMLCapability(root, CRS, version));
            while (ite.hasNext()) {
                Layer layer = ite.next();
                if (layer.getSource() != null) {
                    project.addContent(layer.getXMLCapability(this.getClass().getResourceAsStream(path), version));
                }
            }
            return project;
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return null;
    }

    /**
     * Calcula el bounding box del proyecto con base en los bounding box de la capas que 
     * contiene
     * @return
     */
    public BoundingBox getBoundingBox() {
        BoundingBox bBox = new BoundingBox(Double.MAX_VALUE, Double.MAX_VALUE, Double.MIN_VALUE, Double.MIN_VALUE);
        Iterator<Layer> ite = this.getLayers().iterator();
        while (ite.hasNext()) {
            Layer layer = ite.next();
            if (layer.getSource() != null) {
                if (layer.getXmax() > bBox.getMaxx()) {
                    bBox.setMaxx(layer.getXmax());
                }

                if (layer.getXmin() < bBox.getMinx()) {
                    bBox.setMinx(layer.getXmin());
                }

                if (layer.getYmax() > bBox.getMaxy()) {
                    bBox.setMaxy(layer.getYmax());
                }

                if (layer.getYmin() < bBox.getMiny()) {
                    bBox.setMiny(layer.getYmin());
                }
            }
        }
        return bBox;
    }

    /**
     * Conjuto de los origenes de datos asociados al proyecto
     * @return
     */
    public Set getSources() {
        return sources;
    }

    /**
     * Conjuto de los origenes de datos asociados al proyecto 
     * @param sources
     */
    public void setSources(Set sources) {
        this.sources = sources;
    }

    /**
     * Conjuto de las configuraciones de geocoders asociadas al proyecto
     * @return
     */
    public Set getGeocoderConfigs() {
        return geocoderConfigs;
    }

    /**
     * Conjuto de las configuraciones de geocoders asociadas al proyecto 
     * @param geocoderConfigs
     */
    public void setGeocoderConfigs(Set geocoderConfigs) {
        this.geocoderConfigs = geocoderConfigs;
    }

    /**
     * Listado de los geocoder asociados al proyecto, requiere que previa llamada
     * a loadGeocoders.
     * @return
     * @throws java.lang.IllegalStateException
     */
    public Geocoder[] getGeocoders() throws IllegalStateException {
        if (geocoders == null) {
            throw new IllegalStateException("call load geocoders first");
        }
        return geocoders;
    }
}
