package org.atlas.hibernatedata;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.io.WKBReader;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import org.atlas.service.Feature;

/**
 * Representa un origen de datos del sistema, una relación de objetos geométricos
 * y datos alfanumericos asociados.
 * @author Mario
 */
public class Source {

    private int codesour;
    private String tablsour;
    private String fielsour;
    private Project project;
    private Set layers = new HashSet();
    private double xmin;            //BoundingBox
    private double xmax;            //BoundingBox
    private double ymin;            //BoundingBox
    private double ymax;            //BoundingBox
    private List<Integer> fieldTypes = new ArrayList<Integer>();
    private List<String> fieldNames = new ArrayList<String>();
    private Feature[] features = new Feature[0];
    private Envelope[] bboxes = new Envelope[0];
    private String geomType;

    /**
     * Crea un objeto sin valores
     */
    public Source() {

    }

    @Override
    public String toString() {
        return getTablsour() + " - " + getFielsour();
    }

    /**
     * Carga en memoria los objetos geométricos del origen de datos
     * @param con
     */
    public void loadFeatures(Connection con) {
        if (features.length == 0) {
            if (getFielsour() != null) {
                Statement sta = null;
                ResultSet res = null;
                try {
                    String query = "SELECT count(*) as c FROM \"" + getTablsour() + "\"";
                    sta = con.createStatement();
                    System.out.println(query);
                    res = sta.executeQuery(query);
                    res.next();
                    int capa = res.getInt("c");

                    features = new Feature[capa];
                    bboxes = new Envelope[capa];
                    StringBuilder sql = new StringBuilder();

                    boolean postgis = false;
                    if (org.atlas.util.PostGis.isPostgisInstalled(sta)) {
                        if (org.atlas.util.PostGis.isScriptPresent(sta)) {
                            postgis = true;
                        }
                    }

                    sql.append("SELECT atlas27_id, ");

                    if (postgis) {
                        sql.append("asbinary(" + getFielsour() + ") as " + getFielsour() + " ");
                    } else {
                        sql.append(getFielsour());
                    }

                    //TODO nombre de la etiqueta
                    sql.append(" FROM \"" + getTablsour() + "\"");

                    sta = con.createStatement();
                    res = sta.executeQuery(sql.toString());
                    WKBReader reader = new WKBReader();

                    for (int i = 0; res.next(); i++) {
                        features[i] = new Feature(reader.read(res.getBytes(getFielsour())), res.getInt("atlas27_id"));
                        bboxes[i] = features[i].getGeom().getEnvelopeInternal();
                    }


                } catch (Exception ex) {
                    ex.printStackTrace();
                    this.features = null;
                } finally {
                    try {
                        sta.close();
                        res.close();
                    } catch (SQLException ex) {
                        ex.printStackTrace();
                    }
                }
            }
        }
    }

    /**
     * Calcula el bounding box el origen de datos dada su geometría
     */
    public void calcularDimensiones() {
        if (this.features != null) {
            if (this.features.length > 0) {
                xmin = Double.MAX_VALUE;
                xmax = Double.MIN_VALUE;
                ymin = Double.MAX_VALUE;
                ymax = Double.MIN_VALUE;

                for (int i = 0; i < this.features.length; i++) {
                    Feature f = this.features[i];
                    Geometry g = f.getGeom();
                    Geometry g1 = g.getEnvelope();

                    Coordinate[] coord = g1.getCoordinates();
                    for (int j = 0; j < coord.length; j++) {
                        if (coord[j].x < getXmin()) {
                            xmin = coord[j].x;
                        }
                        if (coord[j].x > getXmax()) {
                            xmax = coord[j].x;
                        }
                        if (coord[j].y < getYmin()) {
                            ymin = coord[j].y;
                        }
                        if (coord[j].y > getYmax()) {
                            ymax = coord[j].y;
                        }
                    }
                }
            }
        }

    }

    /**
     * Obtiene la clase geométrica del origen
     * @return
     */
    public Class getGeomClass() {
        try {
            return Class.forName(this.getGeomType());
        } catch (ClassNotFoundException ex) {
            ex.printStackTrace();
            return null;
        }
    }

    /**
     * Carga los metadatos del origen, consistente en los nombre y tipos de los
     * campos alfanumericos que acompañan a los objetos geométricos
     * @param conn
     */
    public void loadFieldMetadata(Connection conn) {
        try {
            fieldNames.clear();
            fieldTypes.clear();
            ResultSet rs = conn.getMetaData().getColumns(conn.getCatalog(), null, getTablsour(), "%");
            while (rs.next()) {
                if (!this.getFielsour().equals(rs.getString(4)) && !rs.getString(4).equals("atlas27_id")) {
                    fieldNames.add(rs.getString(4));
                    fieldTypes.add(rs.getInt(5));
                }
            }
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
    }

    /**
     * Listado de los tipos de los datos alfanumericos que acompañan a los objetos geométricos
     * @return
     */
    public List<Integer> getFieldTypes() {
        return fieldTypes;
    }

    /**
     * Listado de los nombres de los datos alfanumericos que acompañan a los objetos geométricos 
     * @return
     */
    public List<String> getFieldNames() {
        return fieldNames;
    }

    /**
     * Código del objeto en el sistema de persistencia
     * @return
     */
    public int getCodesour() {
        return codesour;
    }

    /**
     * Código del objeto en el sistema de persistencia 
     * @param codesour
     */
    public void setCodesour(int codesour) {
        this.codesour = codesour;
    }

    /**
     * Nombre de la relación que guarda los datos del origen
     * @return
     */
    public String getTablsour() {
        return tablsour;
    }

    /**
     * Nombre de la relación que guarda los datos del origen 
     * @param tablsour
     */
    public void setTablsour(String tablsour) {
        this.tablsour = tablsour;
    }

    /**
     * Nombre del campo en la relación que guarda la geometría del origen
     * @return
     */
    public String getFielsour() {
        return fielsour;
    }

    /**
     * Nombre del campo en la relación que guarda la geometría del origen 
     * @param fielsour
     */
    public void setFielsour(String fielsour) {
        this.fielsour = fielsour;
    }

    /**
     * Proyecto al que pertenece el origen
     * @return
     */
    public Project getProject() {
        return project;
    }

    /**
     * Proyecto al que pertenece el origen 
     * @param project
     */
    public void setProject(Project project) {
        this.project = project;
    }

    /**
     * Capas en las que se usa el origen
     * @return
     */
    public Set getLayers() {
        return layers;
    }

    /**
     * Capas en las que se usa el origen 
     * @param layers
     */
    public void setLayers(Set layers) {
        this.layers = layers;
    }

    /**
     * Bounding box del origen
     * @return
     */
    public double getXmin() {
        return xmin;
    }

    /**
     * Bounding box del origen 
     * @param xmin
     */
    public void setXmin(double xmin) {
        this.xmin = xmin;
    }

    /**
     * Bounding box del origen
     * @return
     */
    public double getXmax() {
        return xmax;
    }

    /**
     * Bounding box del origen 
     * @param xmax
     */
    public void setXmax(double xmax) {
        this.xmax = xmax;
    }

    /**
     * Bounding box del origen 
     * @return
     */
    public double getYmin() {
        return ymin;
    }

    /**
     * Bounding box del origen 
     * @param ymin
     */
    public void setYmin(double ymin) {
        this.ymin = ymin;
    }

    /**
     * Bounding box del origen 
     * @return
     */
    public double getYmax() {
        return ymax;
    }

    /**
     * Bounding box del origen 
     * @param ymax
     */
    public void setYmax(double ymax) {
        this.ymax = ymax;
    }

    /**
     * Array con los objetos geométricos del origen
     * @return
     */
    public Feature[] getFeatures() {
        return features;
    }

    /**
     * Array con los objetos geométricos del origen 
     * @param features
     */
    public void setFeatures(Feature[] features) {
        this.features = features;
    }

    /**
     * Array con los bounding box de cada geometría en el origen 
     * @return
     */
    public Envelope[] getBboxes() {
        return bboxes;
    }

    /**
     * Array con los bounding box de cada geometría en el origen  
     * @param bboxes
     */
    public void setBboxes(Envelope[] bboxes) {
        this.bboxes = bboxes;
    }

    /**
     * Nombre del tipo de la geometría de origen
     * @return
     */
    public String getGeomType() {
        return geomType;
    }

    /**
     * Nombre del tipo de la geometría de origen 
     * @param geomType
     */
    public void setGeomType(String geomType) {
        this.geomType = geomType;
    }
}
