package org.atlas.hibernatedata;

import java.awt.Font;
import java.util.HashSet;
import java.util.Set;
import org.hibernate.classic.Session;

/**
 * Representa una simbología que detemina los estilos y reglas que se aplicacarán 
 * al renderizar una capa
 * @author Mario
 */
public class Symbology implements Cloneable {

    public static final int TYPE_FIXED_SYMB = 1;
    public static final int TYPE_CLASS_SYMB = 2;
    public static final int TYPE_RANGE_SYMB = 3;
    
    public static final int PRIORITY_LOW = 1;
    public static final int PRIORITY_NORMAL = 2;
    public static final int PRIORITY_HIGH = 3;
    
    private int codesymb;
    private Layer layer;
    private String ruleFieldName = "";
    private String labelFieldName = "";
    private Set rules = new HashSet();
    private int typeSymb;
    private String fontName;
    private int fontStyle;
    private int fontSize;
    private double maxScale;
    private boolean labeled;
    private int priority;
    
    /**
     * Crea un objeto con valores vacios
     */
    public Symbology() {
        priority = PRIORITY_NORMAL;
    }

    /**
     * Identificador del objeto en el modelo de persistencia
     * @return
     */
    public int getCodesymb() {
        return codesymb;
    }

    /**
     * Identificador del objeto en el modelo de persistencia 
     * @param codesymb
     */
    public void setCodesymb(int codesymb) {
        this.codesymb = codesymb;
    }

    /**
     * Capa a la que pertenece la simbología
     * @return
     */
    public Layer getLayer() {
        return layer;
    }

    /**
     * Capa a la que pertenece la simbología 
     * @param layer
     */
    public void setLayer(Layer layer) {
        this.layer = layer;
    }

    /**
     * Nombre del campo en el origen de datos que se usará para derteminar
     * la aplicación de las reglas
     * @return
     */
    public String getRuleFieldName() {
        return ruleFieldName;
    }

    /**
     * Nombre del campo en el origen de datos que se usará para derteminar
     * la aplicación de las reglas
     * @param fieldName
     */
    public void setRuleFieldName(String fieldName) {
        this.ruleFieldName = fieldName;
    }

    /**
     * Conjunto de reglas de la simbología
     * @return
     */
    public Set getRules() {
        return rules;
    }

    /**
     * Conjunto de reglas de la simbología 
     * @param rules
     */
    public void setRules(Set rules) {
        this.rules = rules;
    }

    /**
     * Tipo de la simbología, según las constantes de la clase
     * @return
     */
    public int getTypeSymb() {
        return typeSymb;
    }

    /**
     * Tipo de la simbología, según las constantes de la clase 
     * @param typeSymb
     */
    public void setTypeSymb(int typeSymb) {
        this.typeSymb = typeSymb;
    }

    @Override
    public Object clone() throws CloneNotSupportedException {
        Symbology symb = new Symbology();
        symb.setRuleFieldName(getRuleFieldName());

        symb.setLabelFieldName(getLabelFieldName());
        symb.setFontSize(getFontSize());
        symb.setMaxScale(getMaxScale());
        symb.setFontName(getFontName());
        symb.setFontStyle(getFontStyle());
        symb.setLabeled(isLabeled());

        symb.setLayer(getLayer());
        symb.setTypeSymb(getTypeSymb());
        symb.rules = new HashSet();
        Object[] irules = getRules().toArray();
        for (int i = 0; i < irules.length; i++) {
            Rule r = (Rule) ((Rule) irules[i]).clone();
            r.setSymbology(symb);
            symb.rules.add(r);
        }
        return symb;
    }

    public static void attachDefaulFont(Symbology symbology) {
        symbology.setLabeled(false);
        symbology.setFontName("serif");
        symbology.setFontStyle(Font.PLAIN);
        symbology.setFontSize(10);
        symbology.setMaxScale(-1);
        symbology.setPriority(PRIORITY_NORMAL);
    }

    /**
     * Elimina este objeto del almacenamiento persistente
     * @param sess
     * @throws java.lang.Exception
     */
    public void delete(Session sess) throws Exception {
        sess.update(this);
        Object[] irules = getRules().toArray();
        for (int i = 0; i < irules.length; i++) {
            Rule rule = (Rule) irules[i];
            sess.update(rule);
            Style styl = rule.getStyle();
            sess.update(styl);
            getRules().remove(rule);
            sess.delete(rule);
            if (styl != null) {
                rule.setStyle(null);

                if (styl.getFillcolo() != null) {
                    sess.delete(styl.getFillcolo());
                }

                if (styl.getLinecolo() != null) {
                    sess.delete(styl.getLinecolo());
                }

                sess.delete(styl.getLinestyl());
                sess.delete(styl);
            }
        }
        sess.delete(this);
    }

    /**
     * Almacena este objeto del almacenamiento persistente 
     * @param sess
     * @throws java.lang.Exception
     */
    public void save(Session sess) throws Exception {
        sess.save(this);
        Object[] irules = getRules().toArray();
        for (int i = 0; i < irules.length; i++) {
            Rule rule = (Rule) irules[i];
            Style styl = rule.getStyle();

            if (styl.getFillcolo() != null) {
                sess.save(styl.getFillcolo());
            }

            if (styl.getLinecolo() != null) {
                sess.save(styl.getLinecolo());
            }

            sess.save(styl.getLinestyl());

            sess.save(rule);
            sess.save(styl);
        }
    }

    public String getLabelFieldName() {
        return labelFieldName;
    }

    public void setLabelFieldName(String labelFieldName) {
        this.labelFieldName = labelFieldName;
    }

    public String getFontName() {
        return fontName;
    }

    public void setFontName(String fontName) {
        this.fontName = fontName;
    }

    public boolean isLabeled() {
        return labeled;
    }

    public void setLabeled(boolean labeled) {
        this.labeled = labeled;
    }

    public int getFontStyle() {
        return fontStyle;
    }

    public void setFontStyle(int fontStyle) {
        this.fontStyle = fontStyle;
    }

    public int getFontSize() {
        return fontSize;
    }

    public void setFontSize(int fontSize) {
        this.fontSize = fontSize;
    }

    public double getMaxScale() {
        return maxScale;
    }

    public void setMaxScale(double maxScale) {
        this.maxScale = maxScale;
    }

    public int getPriority() {
        return priority;
    }

    public void setPriority(int priority) {
        this.priority = priority;
    }
}
