package org.atlas.svg;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.InputStream;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;

public class SimpleSvgReader {

    private java.awt.geom.GeneralPath[] paths;
    private SvgStyle[] styles;
    private Rectangle2D rectangle;

    public SimpleSvgReader(InputStream is) throws Exception {

        SAXBuilder builder = new SAXBuilder();
        builder.setEntityResolver(new EntityResolver() {

            public InputSource resolveEntity(String publicId, String systemId) {
                return new InputSource(new StringReader(""));
            }
        });

        Document doc = builder.build(is);
        Element svg = doc.getRootElement();

        paths = new GeneralPath[svg.getChildren().size()];
        styles = new SvgStyle[svg.getChildren().size()];

        for (int i = 0; i < paths.length; i++) {
            Element e = (Element) svg.getChildren().get(i);
            styles[i] = SvgStyle.readStyle(e);
            String data = e.getAttributeValue("d");
            if (styles[i].getFill_rule() != null) {
                if (styles[i].getFill_rule() == GeneralPath.WIND_EVEN_ODD) {
                    paths[i] = new java.awt.geom.GeneralPath(GeneralPath.WIND_EVEN_ODD);
                } else if (styles[i].getFill_rule() == GeneralPath.WIND_NON_ZERO) {
                    paths[i] = new java.awt.geom.GeneralPath(GeneralPath.WIND_NON_ZERO);
                }
            } else {
                paths[i] = new java.awt.geom.GeneralPath();
            }

            GeneralPath path = paths[i];
            List coms = new ArrayList();
            if (data.length() > 0) {
                String tmp = data.substring(0, 1);
                for (int j = 1; j < data.length(); j++) {
                    String chr = data.substring(j, j + 1);
                    if (chr.matches("[MZLCQ]")) {
                        coms.add(tmp);
                        tmp = chr;
                    } else {
                        tmp += chr;
                    }
                }
                coms.add(tmp);
            }
            for (int j = 0; j < coms.size(); j++) {
                String com = ((String) coms.get(j)).trim();
                if (com.startsWith("Z")) {
                    path.closePath();
                } else {
                    Point2D[] points = SvgReader.getPoints(com.substring(1).trim());
                    if (com.startsWith("M")) {
                        path.moveTo(points[0].getX(), points[0].getY());
                    } else if (com.startsWith("L")) {
                        path.lineTo(points[0].getX(), points[0].getY());
                    } else if (com.startsWith("C")) {
                        Point2D[] cp = new Point2D[4];
                        cp[1] = points[0]; //first control Point2D
                        cp[2] = points[1]; //second control Point2D
                        cp[3] = points[2]; //end Point2D
                        path.curveTo(cp[1].getX(), cp[1].getY(), cp[2].getX(), cp[2].getY(), cp[3].getX(), cp[3].getY());
                    }
                }
            }
        }
        rectangle = getSize(paths, styles);
    }

    private static Rectangle2D getSize(GeneralPath[] paths, SvgStyle[] styles) {
        Rectangle2D r = new Rectangle2D.Double(0, 0, 0, 0);
        for (int i = 0; i < paths.length; i++) {
            Rectangle2D r1 = paths[i].getBounds2D();

            try {
                double w = styles[i].getStroke_width();
                r1.setRect(r1.getX(), r1.getY(), r1.getWidth() + w, r1.getHeight() + w);
            } catch (Exception ex) {
            }

            r.add(r1);
        }
        return r;
    }

    public BufferedImage drawGraphics(boolean antialias, Color fill, Color stroke, int maxWidth, int maxHeight) throws IllegalStateException {

        double hz = (double) maxWidth / rectangle.getBounds().getWidth();
        double vr = (double) maxHeight / rectangle.getBounds().getHeight();

        double ct;
        if (hz < vr) {//escalar al ancho
            ct = (double) maxWidth / rectangle.getBounds().getWidth();
        } else {    //escalar al alto
            ct = (double) maxHeight / rectangle.getBounds().getHeight();
        }

        AffineTransform t = new AffineTransform();
        t.setToScale(ct, ct);

        BufferedImage img = new BufferedImage((int) Math.ceil(rectangle.getWidth() * ct), (int) Math.ceil(rectangle.getHeight() * ct), BufferedImage.TYPE_INT_ARGB);
        Graphics2D g = (Graphics2D) img.getGraphics();
        if (antialias) {
            g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        }

        for (int i = 0; i < paths.length; i++) {
            GeneralPath p;

            p = (GeneralPath) paths[i].clone();
            p.transform(t);

            if (styles[i].getStroke_width() != null) {
                g.setStroke(new BasicStroke((float) (styles[i].getStroke_width().floatValue() * ct)));
            } else {
                g.setStroke(new BasicStroke(1));
            }

            if (styles[i].getFill() != null && !styles[i].isFillNone()) {
                if (fill == null) {
                    g.setColor(styles[i].getFill());
                } else {
                    if (isDark(styles[i].getFill())) {
                        g.setColor(stroke);
                    }else{
                        g.setColor(fill);
                    }
                }
                g.fill(p);
            }

            if (styles[i].getStroke() != null) {
                if (stroke == null) {
                    g.setColor(styles[i].getStroke());
                } else {
                    if (isDark(styles[i].getStroke())) {
                        g.setColor(stroke);
                    }else{
                        g.setColor(fill);
                    }
                }
                g.draw(p);
            }

        }
        return img;
    }

    private boolean isDark(Color c) {
        return ((c.getRed() + c.getGreen() + c.getBlue()) / 3) < 155;
    }
}
