
package org.atlas.svg;


import java.awt.geom.AffineTransform;
import java.awt.geom.Arc2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.PathIterator;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Point2D;
import java.awt.geom.RoundRectangle2D;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.StringReader;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.ArrayList;
import java.util.List;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;

public class SvgReader {
    
    private String xml = null;
           
    public static String simplifySVG(InputStream is, int width) throws FileNotFoundException, Exception{
        SAXBuilder builder = new SAXBuilder();
        builder.setEntityResolver(new EntityResolver(){
            public InputSource resolveEntity(String publicId, String systemId) {
                return new InputSource(new StringReader(""));
            }
        });
        
        Document doc = builder.build(is);
        Element svg = doc.getRootElement();
        ArrayList paths = new ArrayList();
        openGroup(svg,paths);
        normalizeSvgGraphics(paths,width);
        return getXML(paths);
    }
    
    private static abstract class SvgGraphic{
        protected GeneralPath     path;
        protected AffineTransform transformation;
        protected SvgStyle           style;
        
        public abstract void findPolygons(String data);
        
        public GeneralPath getPath() {
            return path;
        }
        
        public AffineTransform getTransformation() {
            return transformation;
        }
        
        public SvgStyle getStyle() {
            return style;
        }
        
        public String getXML(){
            PathIterator ite = path.getPathIterator(null);
            String sdata = "";
            DecimalFormat d = new DecimalFormat();
            
            DecimalFormatSymbols unusualSymbols = new DecimalFormatSymbols();
            unusualSymbols.setDecimalSeparator('.');
            d.setDecimalFormatSymbols(unusualSymbols);
            d.setMaximumFractionDigits(4);
            d.setGroupingUsed(false);
            while(!ite.isDone()){
                double [] data = new double[6];
                switch(ite.currentSegment(data)){
                    case PathIterator.SEG_MOVETO:
                        sdata+=("M"+d.format(data[0])+","+d.format(data[1]));
                        break;
                    case PathIterator.SEG_LINETO:
                        sdata+=("L"+d.format(data[0])+","+d.format(data[1]));
                        break;
                    case PathIterator.SEG_QUADTO:
                        sdata+=("Q"+d.format(data[0])+","+d.format(data[1])+" "+d.format(data[2])+","+d.format(data[3])+" ");
                        break;
                    case PathIterator.SEG_CUBICTO:
                        sdata+=("C"+d.format(data[0])+","+d.format(data[1])+" "+d.format(data[2])+","+d.format(data[3])+" "+d.format(data[4])+","+d.format(data[5]));
                        break;
                    case PathIterator.SEG_CLOSE:
                        sdata+=("Z");
                        break;
                }
                ite.next();
            }
            
            String fill;
            String stroke;
            
            if(style.isFillNone()){
                fill = "none";
            }else if(style.getFill()!=null){
                fill = "rgb("+style.getFill().getRed()+","+style.getFill().getGreen()+","+style.getFill().getBlue()+")";
            }else{
                fill = "rgb(0, 0, 0)";
            }
            
            
            if(style.isStrokeNone()){
                stroke = "none";
            }else if(style.getStroke()!=null){
                stroke = "rgb("+style.getStroke().getRed()+","+style.getStroke().getGreen()+","+style.getStroke().getBlue()+")";
            }else{
                stroke = "none";
            }
            
            String stroke_w = "";
            if(style.getStroke_width()!=null){
                
                Point2D r1 = new Point2D.Double();
                transformation.transform(new Point2D.Double(0,0), r1);
                
                Point2D r2 = new Point2D.Double();
                transformation.transform(new Point2D.Double(10,10), r2);
                
                double d1 = Math.sqrt(200);
                double d2 = Math.sqrt(Math.pow(r1.getX()-r2.getX(), 2)+Math.pow(r1.getY()-r2.getY(), 2));
                
                double scal = d2/d1;
                double w = style.getStroke_width().doubleValue();
                double gross = w*scal;
                stroke_w = "stroke-width=\""+gross+"\"";
            }
            
            String fill_rule;
            if(style.getFill_rule()!=null){
                if(style.getFill_rule().equals(GeneralPath.WIND_EVEN_ODD)){
                    fill_rule = "evenodd";
                }else{
                    fill_rule = "nonzero";
                }
            }else{
                fill_rule = "nonzero";
            }
            
            return "<path fill-rule=\""+fill_rule+"\" "+stroke_w+" fill=\""+fill+"\"  stroke=\""+stroke+"\" d=\""+sdata+"\"/>";
        }
    }
    
    private static class SvgLine extends SvgGraphic{
        public SvgLine(String data, String transformations, SvgStyle style){
            this.style = style;
            transformation = getTransformMatrix(transformations);
            findPolygons(data);
        }
        public void findPolygons(String data){
            path = new GeneralPath();
            double[] nums = getNumberList(data);
            double a = nums[0];
            double b = nums[1];
            double c = nums[2];
            double d = nums[3];
            
            java.awt.geom.Line2D ln = new java.awt.geom.Line2D.Double(a, b, c, d);
            getPath().append(ln.getPathIterator(null),true);
            getPath().transform(getTransformation());
        }
    }
    
    private static class SvgPath extends SvgGraphic{
        
        public SvgPath(String data, String transformations, SvgStyle style){
            this.style = style;
            transformation = getTransformMatrix(transformations);
            findPolygons(data);
        }
        
        public double calcVectorAngle(double ux, double uy, double vx, double vy){
            double ta = Math.atan2(uy, ux);
            double tb = Math.atan2(vy, vx);
            if (tb >= ta)
                return tb-ta;
            return (Math.PI*2) - (ta-tb);
        }
        
        public void findPolygons(String data){
            path = new java.awt.geom.GeneralPath();
            ArrayList coms = new ArrayList();
            
            if(data.length()>0){
                String tmp = data.substring(0,1);
                for (int i = 1; i < data.length(); i++) {
                    String chr = data.substring(i,i+1);
                    if(chr.matches("[MmZzLlHhVvCcSsQqTtAa]")){
                        coms.add(tmp);
                        tmp=chr;
                    }else{
                        tmp+=chr;
                    }
                }
                coms.add(tmp);
            }
            for (int i = 0; i < coms.size(); i++) {
                String com = ((String)coms.get(i)).trim();
                
                if(com.startsWith("z")||com.startsWith("Z")){
                    this.getPath().closePath();
                }else{
                    Point2D[] points = null;
                    try {
                        boolean ban = false;
                        if(com.startsWith("M")){
                            points = getPoints(com.substring(1).trim());
                            ban = true;
                            this.getPath().moveTo(points[0].getX(), points[0].getY());
                            for (int j = 1; j < points.length; j++) {
                                this.getPath().lineTo(points[j].getX(), points[j].getY());
                            }
                        }
                        
                        if(com.startsWith("m")){
                            points = getPoints(com.substring(1).trim());
                            ban = true;
                            double curx = getPath().getCurrentPoint().getX();
                            double cury = getPath().getCurrentPoint().getY();
                            this.getPath().moveTo(points[0].getX()+curx, points[0].getY()+cury);
                            for (int j = 1; j < points.length; j++) {
                                curx = getPath().getCurrentPoint().getX();
                                cury = getPath().getCurrentPoint().getY();
                                this.getPath().lineTo(points[j].getX()+curx, points[j].getY()+cury);
                                
                            }
                        }
                        
                        if(com.startsWith("L")){
                            points = getPoints(com.substring(1).trim());
                            ban = true;
                            for (int j = 0; j < points.length; j++) {
                                this.getPath().lineTo(points[j].getX(), points[j].getY());
                            }
                        }
                        
                        if(com.startsWith("l")){
                            points = getPoints(com.substring(1).trim());
                            ban = true;
                            for (int j = 0; j < points.length; j++) {
                                double curx = getPath().getCurrentPoint().getX();
                                double cury = getPath().getCurrentPoint().getY();
                                this.getPath().lineTo(points[j].getX()+curx, points[j].getY()+cury);
                            }
                        }
                        
                        if(com.startsWith("C")){
                            points = getPoints(com.substring(1).trim());
                            ban = true;
                            for (int j = 0; j < points.length; j+=3) {
                                
                                Point2D[] cp = new Point2D[4];
                                cp[1] = points[0+j]; //first control Point2D
                                cp[2] = points[1+j]; //second control Point2D
                                cp[3] = points[2+j]; //end Point2D
                                
                                double curx = getPath().getCurrentPoint().getX();
                                double cury = getPath().getCurrentPoint().getY();
                                getPath().curveTo(cp[1].getX(), cp[1].getY(), cp[2].getX(), cp[2].getY(), cp[3].getX(), cp[3].getY());
                            }
                        }
                        
                        if(com.startsWith("c")){
                            points = getPoints(com.substring(1).trim());
                            ban = true;
                            for (int j = 0; j < points.length; j+=3) {
                                double curx = getPath().getCurrentPoint().getX();
                                double cury = getPath().getCurrentPoint().getY();
                                Point2D[] cp = new Point2D[4];
                                cp[1] = points[0+j]; //first control Point2D
                                cp[2] = points[1+j]; //second control Point2D
                                cp[3] = points[2+j]; //end Point2D
                                getPath().curveTo(cp[1].getX()+curx, cp[1].getY()+cury, cp[2].getX()+curx, cp[2].getY()+cury, cp[3].getX()+curx, cp[3].getY()+cury);
                            }
                        }
                        
                        
                        if(com.startsWith("s")){
                            points = getPoints(com.substring(1).trim());
                            ban = true;
                            for (int j = 0; j < points.length; j+=2) {
                                double curx = getPath().getCurrentPoint().getX();
                                double cury = getPath().getCurrentPoint().getY();
                                PathIterator ite = getPath().getPathIterator(null);
                                double [] dat = new double[6];
                                int type = -1;
                                while(!ite.isDone()){
                                    type = ite.currentSegment(dat);
                                    ite.next();
                                }
                                Point2D[] cp = new Point2D[4];
                                if(type==PathIterator.SEG_CUBICTO){
                                    double rx = 2*curx-dat[2];
                                    double ry = 2*cury-dat[3];
                                    cp[1] = new Point2D.Double(rx, ry); //first control Point2D
                                }else{
                                    cp[1] = new Point2D.Double(curx, cury); //first control Point2D
                                }
                                cp[2] = points[0+j]; //second control Point2D
                                cp[3] = points[1+j]; //end Point2D
                                getPath().curveTo(cp[1].getX(), cp[1].getY(), cp[2].getX()+curx, cp[2].getY()+cury, cp[3].getX()+curx, cp[3].getY()+cury);
                            }
                        }
                        
                        if(com.startsWith("S")){
                            points = getPoints(com.substring(1).trim());
                            ban = true;
                            for (int j = 0; j < points.length; j+=2) {
                                double curx = getPath().getCurrentPoint().getX();
                                double cury = getPath().getCurrentPoint().getY();
                                PathIterator ite = getPath().getPathIterator(null);
                                double [] dat = new double[6];
                                int type = -1;
                                while(!ite.isDone()){
                                    type = ite.currentSegment(dat);
                                    ite.next();
                                }
                                Point2D[] cp = new Point2D[4];
                                if(type==PathIterator.SEG_CUBICTO){
                                    double rx = 2*curx-dat[2];
                                    double ry = 2*cury-dat[3];
                                    cp[1] = new Point2D.Double(rx, ry); //first control Point2D
                                }else{
                                    cp[1] = new Point2D.Double(curx, cury); //first control Point2D
                                }
                                cp[2] = points[0+j]; //second control Point2D
                                cp[3] = points[1+j]; //end Point2D
                                getPath().curveTo(cp[1].getX(), cp[1].getY(), cp[2].getX(), cp[2].getY(), cp[3].getX(), cp[3].getY());
                            }
                        }
                        
                        if(com.startsWith("A")){
                            ban = true;
                            double curx = getPath().getCurrentPoint().getX();
                            double cury = getPath().getCurrentPoint().getY();
                            double[] nums = getNumberList(com.substring(1).trim());
                            double x1 = curx;
                            double y1 = cury;
                            double x2 = nums[5];
                            double y2 = nums[6];
                            double ang1 = Math.toRadians(nums[2]);
                            double rx    = nums[0];
                            double ry    = nums[1];
                            boolean fa = true;
                            if(nums[3]==0)fa = false;
                            boolean fs = true;
                            if(nums[4]==0)fs = false;
                            boolean valid = true;
                            
                            if (x1 == x2 && y1 == y2) valid = false;
                            if (rx == 0.0 || ry == 0.0) valid = false;
                            if (rx<0.0) rx = -rx;
                            if (ry<0.0) ry = -ry;
                            
                            if(valid){
                                
                                double cosPhi = Math.cos(ang1);
                                double sinPhi = Math.sin(ang1);
                                
                                double x1dash =  cosPhi * (x1-x2)/2.0 + sinPhi * (y1-y2)/2.0;
                                double y1dash = -sinPhi * (x1-x2)/2.0 + cosPhi * (y1-y2)/2.0;
                                
                                double root;
                                double numerator = rx*rx*ry*ry - rx*rx*y1dash*y1dash - ry*ry*x1dash*x1dash;
                                if (numerator < 0.0) {
                                    double s = Math.sqrt(1.0 - numerator/(rx*rx*ry*ry));
                                    rx *= s;
                                    ry *= s;
                                    root = 0.0;
                                }else {
                                    if(fa == fs){
                                        
                                        root=-Math.sqrt( numerator/(rx*rx*y1dash*y1dash+ry*ry*x1dash*x1dash));
                                    }else{
                                        root= Math.sqrt( numerator/(rx*rx*y1dash*y1dash+ry*ry*x1dash*x1dash));
                                    }
                                }
                                
                                double cxdash = root*rx*y1dash/ry;
                                double cydash = -root*ry*x1dash/rx;
                                
                                double cx = cosPhi * cxdash - sinPhi * cydash + (x1+x2)/2.0;
                                double cy = sinPhi * cxdash + cosPhi * cydash + (y1+y2)/2.0;
                                
                                Arc2D arc = new Arc2D.Double();
                                
                                
                                double theta1 = calcVectorAngle(1.0, 0.0, (x1dash-cxdash)/rx, (y1dash-cydash)/ry);
                                double dtheta = calcVectorAngle((x1dash-cxdash)/rx, (y1dash-cydash)/ry,(-x1dash-cxdash)/rx, (-y1dash-cydash)/ry);
                                if (!fs && dtheta>0)
                                    dtheta -= 2.0*Math.PI;
                                else if (fs && dtheta<0)
                                    dtheta += 2.0*Math.PI;
                                
                                // 3. convert into cubic bezier segments <= 90deg
                                int segments = (int)Math.ceil(Math.abs(dtheta/(Math.PI/2.0)));
                                double delta = dtheta/segments;
                                double t = 8.0/3.0 * Math.sin(delta/4.0) * Math.sin(delta/4.0) / Math.sin(delta/2.0);
                                for (int k = 0; k < segments; ++k) {
                                    double cosTheta1 = Math.cos(theta1);
                                    double sinTheta1 = Math.sin(theta1);
                                    double theta2 = theta1 + delta;
                                    double cosTheta2 = Math.cos(theta2);
                                    double sinTheta2 = Math.sin(theta2);
                                    // a) calculate endpoint of the segment:
                                    double xe = cosPhi * rx*cosTheta2 - sinPhi * ry*sinTheta2 + cx;
                                    double ye = sinPhi * rx*cosTheta2 + cosPhi * ry*sinTheta2 + cy;
                                    
                                    // b) calculate gradients at start/end points of segment:
                                    double dx1 = t * ( - cosPhi * rx*sinTheta1 - sinPhi * ry*cosTheta1);
                                    double dy1 = t * ( - sinPhi * rx*sinTheta1 + cosPhi * ry*cosTheta1);
                                    
                                    double dxe = t * ( cosPhi * rx*sinTheta2 + sinPhi * ry*cosTheta2);
                                    double dye = t * ( sinPhi * rx*sinTheta2 - cosPhi * ry*cosTheta2);
                                    
                                    getPath().curveTo(x1+dx1, y1+dy1,xe+dxe,ye+dye, xe, ye);
                                    
                                    // do next segment
                                    theta1 = theta2;
                                    x1 = (float)xe;
                                    y1 = (float)ye;
                                }
                                
                            }//valid
                        }
                        
                        if(!ban){
                            System.out.println(com);
                        }
                    } catch(Exception ex) {
                        System.out.println("Error: "+com);
                    }
                }
            }
            
            getPath().transform(getTransformation());
        }
    }
    
    private static class SvgPolygon extends SvgGraphic{
        public SvgPolygon(String data, String transformations, SvgStyle style){
            this.style = style;
            transformation = getTransformMatrix(transformations);
            findPolygons(data);
        }
        
        public void findPolygons(String data){
            path = new GeneralPath();
            Point2D[] points = getPoints(data);
            
            if(points.length>1){
                getPath().moveTo(points[0].getX(), points[0].getY());
                for (int i = 1; i < points.length; i++) {
                    getPath().lineTo(points[i].getX(), points[i].getY());
                }
            }
            
            getPath().closePath();
            getPath().transform(getTransformation());
        }
    }
    
    private static class SvgCircle extends SvgGraphic{
        public SvgCircle(String data, String transformations, SvgStyle style){
            this.style = style;
            transformation = getTransformMatrix(transformations);
            findPolygons(data);
        }
        
        public void findPolygons(String data){
            path = new GeneralPath();
            double[] nums = getNumberList(data);
            double a = nums[0];
            double b = nums[1];
            double r = nums[2];
            java.awt.geom.Ellipse2D el = new java.awt.geom.Ellipse2D.Double(a-r,b-r,r*2,r*2);
            getPath().append(el.getPathIterator(null),true);
            getPath().transform(getTransformation());
        }
    }
    
    private static class SvgEllipse extends SvgGraphic{
        public SvgEllipse(String data, String transformations, SvgStyle style){
            super();
            this.style = style;
            transformation = getTransformMatrix(transformations);
            findPolygons(data);
        }
        public void findPolygons(String data){
            path = new GeneralPath();
            double[] nums = getNumberList(data);
            double x = nums[0];
            double y = nums[1];
            double a = nums[2];
            double b = nums[3];
            
            x-=a;
            y-=b;
            
            a*=2;
            b*=2;
            
            java.awt.geom.Ellipse2D el = new java.awt.geom.Ellipse2D.Double(x,y,a,b);
            getPath().append(el.getPathIterator(null),true);
            getPath().transform(getTransformation());
        }
    }
    
    private static class SvgRect extends SvgGraphic{
        public SvgRect(String data, String transformations, SvgStyle style){
            super();
            this.style = style;
            transformation = getTransformMatrix(transformations);
            findPolygons(data);
        }
        
        public void findPolygons(String data){
            path = new GeneralPath();
            double[] nums = getNumberList(data);
            double x = nums[0];
            double y = nums[1];
            double ancho = nums[2];
            double alto  = nums[3];
            double rx = nums[4];
            double ry  = nums[5];
            
            //java.awt.geom.Rectangle2D el = new java.awt.geom.Rectangle2D.Double(x,y,ancho,alto);
            
            RoundRectangle2D el = new RoundRectangle2D.Double(x,y,ancho,alto,rx,ry);
            
            getPath().append(el.getPathIterator(null),true);
            getPath().transform(getTransformation());
        }
    }
    
    private static String getGroupTransformations(Element g){
        String transform = g.getAttributeValue("transform");
        if(transform==null)transform = "";
        if(g.getParentElement()==null){
            return transform;
        }else{
            return getGroupTransformations(g.getParentElement())+" "+transform;
        }
    }
    
    private static AffineTransform getTransformMatrix(String tx){
        
        String stmp = "";
        List names  = new ArrayList();
        List params = new ArrayList();
        int sta = 1;
        for (int pos = 0; pos < tx.length(); pos++) {
            String tok = tx.substring(pos,pos+1);
            switch(sta){
                case 1:
                    if(tok.matches("[a-z]")){
                        stmp+=tok;
                    }else if(tok.matches("[(]")){
                        names.add(stmp);
                        stmp = "";
                        sta = 2;
                    }
                    break;
                case 2:
                    if(tok.matches("[)]")){
                        sta = 1;
                        params.add(stmp);
                        stmp = "";
                    }else{
                        stmp+=tok;
                    }
                    break;
            }
        }
        
        AffineTransform tra = new AffineTransform();
        
        for (int i = 0; i < names.size(); i++) {
            double[] nums = getNumberList((String)params.get(i));
            String name = (String)names.get(i);
            if(nums.length>0){
                if(name.equals("matrix")){
                    tra.concatenate(new AffineTransform(nums[0],nums[1],nums[2],nums[3],nums[4],nums[5]));
                } else if(name.equals("translate")){
                    AffineTransform tmp = new AffineTransform();
                    tmp.setToTranslation(nums[0],nums[1]);
                    tra.concatenate(tmp);
                } else if(name.equals("scale")){
                    AffineTransform tmp = new AffineTransform();
                    if(nums.length==1){
                        tmp.setToScale(nums[0],nums[0]);
                    }else{
                        tmp.setToScale(nums[0],nums[1]);
                    }
                    tra.concatenate(tmp);
                } else if(name.equals("rotate")){
                    AffineTransform tmp = new AffineTransform();
                    tmp.setToRotation(Math.toRadians(nums[0]));
                    tra.concatenate(tmp);
                }
                //TODO rotate skewX skewY
            }
        }
        return tra;
    }
    
    private static String getPixelLength(String val){
        val = val.toLowerCase().trim();
        double k = 1;
        if(val.indexOf("pt")>0){
            k = 1.25;
        }else if(val.indexOf("pc")>0){
            k = 15;
        }else if(val.indexOf("mm")>0){
            k = 3.543307;
        }else if(val.indexOf("cm")>0){
            k = 35.43307;
        }else if(val.indexOf("in")>0){
            k = 90;
        }
        
        double[] nums = getNumberList(val);
        if(nums.length>0){
            return (nums[0]*k)+"";
        }else{
            return "0";
        }
    }
    
    private static void openGroup(Element g, List graphics){
        List children = g.getChildren();
        String gTrans = getGroupTransformations(g);
        for (int i = 0; i < children.size(); i++) {
            Element elem = (Element)children.get(i);
            String myTransform = elem.getAttributeValue("transform");
            SvgStyle style = new SvgStyle();
            SvgStyle.getStyle(elem, style);
            String name = elem.getName();
            if(myTransform==null)myTransform = "";
            
            if(name.equals("line")){
                
                String x1 = "0";
                if(elem.getAttributeValue("x1")!=null)x1= getPixelLength(elem.getAttributeValue("x1"));
                
                String y1 = "0";
                if(elem.getAttributeValue("y1")!=null)y1 = getPixelLength(elem.getAttributeValue("y1"));
                
                String x2 = "0";
                if(elem.getAttributeValue("x2")!=null)x2= getPixelLength(elem.getAttributeValue("x2"));
                
                String y2 = "0";
                if(elem.getAttributeValue("y2")!=null)y2 = getPixelLength(elem.getAttributeValue("y2"));
                
                String data =  x1+" "+y1+" "+x2+" "+y2;
                SvgGraphic tmp = new SvgLine(data, gTrans+" "+myTransform, style);
                graphics.add(tmp);
            } else if(name.equals("path")){
                String data =  elem.getAttributeValue("d");
                SvgGraphic tmp = new SvgPath(data, gTrans+" "+myTransform, style);
                graphics.add(tmp);
            } else if(name.equals("polygon")){
                String data =  elem.getAttributeValue("points");
                graphics.add(new SvgPolygon(data, gTrans+" "+myTransform, style));
            } else if(name.equals("circle")){
                String data =  getPixelLength(elem.getAttributeValue("cx"))+" "+getPixelLength(elem.getAttributeValue("cy"))+" "+getPixelLength(elem.getAttributeValue("r"));
                graphics.add(new SvgCircle(data, gTrans+" "+myTransform, style));
            } else if(name.equals("rect")){
                String x = "0";
                if(elem.getAttributeValue("x")!=null)x = getPixelLength(elem.getAttributeValue("x"));
                
                String y = "0";
                if(elem.getAttributeValue("y")!=null)y = getPixelLength(elem.getAttributeValue("y"));
                
                String width = "0";
                if(elem.getAttributeValue("width")!=null)width = getPixelLength(elem.getAttributeValue("width"));
                
                String height = "0";
                if(elem.getAttributeValue("height")!=null)height = getPixelLength(elem.getAttributeValue("height"));
                
                String rx = "0";
                if(elem.getAttributeValue("rx")!=null)rx = getPixelLength(elem.getAttributeValue("rx"));
                
                String ry = "0";
                if(elem.getAttributeValue("ry")!=null)ry = getPixelLength(elem.getAttributeValue("ry"));
                
                String data =  x+" "+y+" "+width+" "+height+" "+rx+" "+ry;
                graphics.add(new SvgRect(data, gTrans+" "+myTransform, style));
            } else if(name.equals("ellipse")){
                String data =  getPixelLength(elem.getAttributeValue("cx"))+" "+getPixelLength(elem.getAttributeValue("cy"))+" "+getPixelLength(elem.getAttributeValue("rx"))+" "+getPixelLength(elem.getAttributeValue("ry"));
                graphics.add(new SvgEllipse(data, gTrans+" "+myTransform, style));
            } else if(name.equals("g")||name.equals("switch")){
                openGroup(elem,graphics);
            } else{
                System.out.println("Ingored arg "+name);
            }
        }
    }
    
    private static String getXML(ArrayList graps){
        Rectangle2D r = getSize(graps);
        int w = (int)r.getWidth();
        int h = (int)r.getHeight();
        
        StringBuilder sb = new StringBuilder("<?xml version=\"1.0\" encoding= \"UTF-8\" standalone=\"yes\"?>");
        sb.append("\n<svg xmlns:svg=\"http://www.w3.org/2000/svg\" xmlns=\"http://www.w3.org/2000/svg\" width = \""+w+"px\" height = \""+h+"px\">");
        for (int i = 0; i < graps.size(); i++) {
            sb.append("\n"+((SvgGraphic)graps.get(i)).getXML());
        }
        sb.append("\n</svg>");
        return sb.toString();
    }
    
    private static double[] getNumberList(String numbers){
        String par=numbers.toLowerCase()+";";
        List nums = new ArrayList();
        int sta = 1;
        String tmp = "";
        for (int pos = 0; pos < par.length(); pos++) {
            String tok = par.substring(pos,pos+1);
            switch(sta){
                case 1:
                    if(tok.matches("[0-9]")){
                        sta = 2;
                        tmp+=tok;
                    } else if(tok.matches("[.]")){
                        sta = 5;
                        tmp+=tok;
                    } else if(tok.matches("[-]")){
                        sta = 3;
                        tmp+=tok;
                    }
                    break;
                case 2:
                    if(tok.matches("[0-9]")){
                        tmp+=tok;
                    } else if(tok.matches("[.]")){
                        tmp+=tok;
                        sta = 5;
                    } else if(tok.matches("[ ,\\-;]")){
                        sta = 1;
                        pos--;
                        nums.add(new Double(Double.parseDouble(tmp)));
                        tmp="";
                    } else if(tok.matches("e")){
                        tmp+=tok;
                        sta = 7;
                    }
                    break;
                case 3:
                    if(tok.matches("[0-9]")){
                        sta = 2;
                        tmp+=tok;
                    }
                    break;
                case 5:
                    if(tok.matches("[0-9]")){
                        sta = 6;
                        tmp+=tok;
                    }
                    break;
                case 6:
                    if(tok.matches("[0-9]")){
                        tmp+=tok;
                    } else if(tok.matches("[e]")){
                        tmp+=tok;
                        sta = 7;
                    } else if(tok.matches("[ ,\\-;]")){
                        sta = 1;
                        pos--;
                        nums.add(new Double(Double.parseDouble(tmp)));
                        tmp="";
                    }
                    break;
                case 7:
                    if(tok.matches("[0-9]")){
                        sta = 8;
                        tmp+=tok;
                    } else if(tok.matches("[.]")){
                        sta = 9;
                        tmp+=tok;
                    } else if(tok.matches("[-]")){
                        sta = 11;
                        tmp+=tok;
                    }
                    break;
                case 8:
                    if(tok.matches("[0-9]")){
                        tmp+=tok;
                    } else if(tok.matches("[ ,\\-;]")){
                        sta = 1;
                        pos--;
                        nums.add(new Double(Double.parseDouble(tmp)));
                        tmp="";
                    }
                    break;
                case 9:
                    if(tok.matches("[0-9]")){
                        tmp+=tok;
                        sta = 10;
                    }
                    break;
                case 10:
                    if(tok.matches("[0-9]")){
                        tmp+=tok;
                    } else if(tok.matches("[ ,\\-;]")){
                        sta = 1;
                        pos--;
                        nums.add(new Double(Double.parseDouble(tmp)));
                        tmp="";
                    }
                    break;
                case 11:
                    if(tok.matches("[0-9]")){
                        sta = 8;
                        tmp+=tok;
                    }
                    break;
            }
        }
        
        double[] res = new double[nums.size()];
        for (int i = 0; i < nums.size(); i++) {
            res[i] = ((Double)nums.get(i)).doubleValue();
        }
        return res;
    }
    
    protected static Point2D[] getPoints(String params){
        double[] nums = getNumberList(params);
        Point2D[] pts = new Point2D[nums.length/2];
        for (int i = 0, j=0; i < nums.length; i+=2, j++) {
            pts[j] = new Point2D.Double(nums[i],nums[i+1]);
        }
        return pts;
    }
    
    private static Rectangle2D getSize(List graphs){
        Rectangle2D r = ((SvgGraphic)graphs.get(0)).getPath().getBounds2D();
        for (int i = 1; i < graphs.size(); i++) {
            Rectangle2D r1 = ((SvgGraphic)graphs.get(i)).getPath().getBounds2D();
            r.add(r1);
        }
        return r;
    }
    
    private static void normalizeSvgGraphics(List graphs, int width){
        Rectangle2D r = getSize(graphs);
        double minX = r.getX();
        double minY = r.getY();
        double scale = (double)width/(r.getWidth());
        java.awt.geom.AffineTransform a = new java.awt.geom.AffineTransform();
        java.awt.geom.AffineTransform b = new java.awt.geom.AffineTransform();
        a.setToScale(scale, scale);
        b.setToTranslation(-minX, -minY);
        a.concatenate(b);
        for (int i = 0; i < graphs.size(); i++) {
            SvgGraphic gra = (SvgGraphic)graphs.get(i);
            gra.getPath().transform(a);
            if(gra.getStyle().getStroke_width()!=null)
                gra.getStyle().setStroke_width(new Double(gra.getStyle().getStroke_width().doubleValue()*scale));
        }
    }
}

